/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSStyleSheet.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSStyleSheet.h"
#include "gdom/GdomStyleSheet.h"
#include "gdom/PrivateGdomCSSStyleSheet.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomStyleSheet.h"
#include "gdom/PrivateIntGdomCSSStyleSheet.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMCSSStyleSheet::GDOMCSSStyleSheet(PassRefPtr<WebCore::CSSStyleSheet> impl)
    : GDOMStyleSheet(impl)
{
}

gulong
gdom_css_style_sheet_insert_rule (GdomCSSStyleSheet *thiz, gchar * rule, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleSheet * item = (gdom_css_style_sheet_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (rule, 0);
    WebCore::String _g_rule = WebCore::String::fromUTF8(rule);
    WebCore::ExceptionCode ec = 0;
    gulong res = item->insertRule(_g_rule, index, ec);
    return res;

}

void
gdom_css_style_sheet_delete_rule (GdomCSSStyleSheet *thiz, gulong index)
{
    g_return_if_fail (thiz);
    WebCore::CSSStyleSheet * item = (gdom_css_style_sheet_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->deleteRule(index, ec);

}

glong
gdom_css_style_sheet_add_rule (GdomCSSStyleSheet *thiz, gchar * selector, gchar * style, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleSheet * item = (gdom_css_style_sheet_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (selector, 0);
    g_return_val_if_fail (style, 0);
    WebCore::String _g_selector = WebCore::String::fromUTF8(selector);
    WebCore::String _g_style = WebCore::String::fromUTF8(style);
    WebCore::ExceptionCode ec = 0;
    glong res = item->addRule(_g_selector, _g_style, index, ec);
    return res;

}

void
gdom_css_style_sheet_remove_rule (GdomCSSStyleSheet *thiz, gulong index)
{
    g_return_if_fail (thiz);
    WebCore::CSSStyleSheet * item = (gdom_css_style_sheet_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->removeRule(index, ec);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CSSStyleSheet.h"
#include <gdom/GdomCSSStyleSheet.h>
#include <gdom/PrivateGdomCSSStyleSheet.h>
#include <gdom/PrivateIntGdomCSSStyleSheet.h>
#include <gdom/GdomCSSStyleSheetprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCSSStyleSheet, gdom_css_style_sheet, GDOM_TYPE_STYLE_SHEET);

/*
struct _GdomCSSStyleSheetPrivate {
    class WebCore::CSSStyleSheet *item;
};

#define GDOM_CSS_STYLE_SHEET_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CSS_STYLE_SHEET, GdomCSSStyleSheetPrivate))
*/
G_END_DECLS
WebCore::CSSStyleSheet *
gdom_css_style_sheet_get_private(GdomCSSStyleSheet *request)
{
    GDOMCSSStyleSheet *res;
    res = static_cast<GDOMCSSStyleSheet*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CSSStyleSheet*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_OWNER_RULE,
    PROP_CSS_RULES,
    PROP_RULES,
};


static void gdom_css_style_sheet_finalize(GObject* object)
{
    /*GdomCSSStyleSheet* request = GDOM_CSS_STYLE_SHEET(object);*/
    /*GdomCSSStyleSheetPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_STYLE_SHEET));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_css_style_sheet_parent_class)->finalize(object);
}

static void gdom_css_style_sheet__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSStyleSheet *self = GDOM_CSS_STYLE_SHEET(object);
    WebCore::CSSStyleSheet * gself = (gdom_css_style_sheet_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_style_sheet__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSStyleSheet *self = GDOM_CSS_STYLE_SHEET(object);
    WebCore::CSSStyleSheet * gself = (gdom_css_style_sheet_get_private(self));
    switch (prop_id) {
    case PROP_OWNER_RULE:
    {
         PassRefPtr<WebCore::CSSRule> ptr = gself->ownerRule();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CSS_RULES:
    {
         PassRefPtr<WebCore::CSSRuleList> ptr = gself->cssRules();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_RULES:
    {
         PassRefPtr<WebCore::CSSRuleList> ptr = gself->rules();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_style_sheet_class_init(GdomCSSStyleSheetClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_css_style_sheet_finalize;
    g_klass->set_property = gdom_css_style_sheet__set_property_fn;
    g_klass->get_property = gdom_css_style_sheet__get_property_fn;

    g_object_class_install_property (g_klass, PROP_OWNER_RULE,
            g_param_spec_object ("owner_rule", /* name */
                "css_style_sheet_owner_rule", /* short description */
                "read-only  GdomCSSRule * CSSStyleSheet.owner_rule", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CSS_RULES,
            g_param_spec_object ("css_rules", /* name */
                "css_style_sheet_css_rules", /* short description */
                "read-only  GdomCSSRuleList * CSSStyleSheet.css_rules", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE_LIST, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RULES,
            g_param_spec_object ("rules", /* name */
                "css_style_sheet_rules", /* short description */
                "read-only  GdomCSSRuleList * CSSStyleSheet.rules", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE_LIST, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCSSStyleSheetPrivate));*/
}

static void gdom_css_style_sheet_init(GdomCSSStyleSheet* request)
{
    /*GdomCSSStyleSheetPrivate* priv = GDOM_CSS_STYLE_SHEET_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

